@@ -1,12 +1,14 @@
 import json
 import sys
+import os
+
+from io import BytesIO
 
 import datashader as ds
 import numpy as np
 import geopandas as gpd
 import dask.array as da
 
-
 import datashader.transfer_functions as tf
 import datashader.reductions as rd
 from datashader.colors import rgb
@@ -21,8 +23,7 @@ from mapshader.mercator import MercatorTileDefinition
 from mapshader.sources import MapSource
 from .multifile import MultiFileRaster
 
-import spatialpandas
-
+import spatialpandas as spd
 
 tile_def = MercatorTileDefinition(x_range=(-20037508.34, 20037508.34),
                                   y_range=(-20037508.34, 20037508.34))
@@ -73,6 +74,7 @@ def create_agg(source: MapSource,
     xfield = source.xfield
     yfield = source.yfield
     zfield = source.zfield
+    geometry_field = source.geometry_field
     agg_func = source.agg_func
     geometry_type = source.geometry_type
 
@@ -92,7 +94,7 @@ def create_agg(source: MapSource,
                     x_range=(xmin, xmax), y_range=(ymin, ymax))
 
     if geometry_type == 'point':
-        return point_aggregation(cvs, dataset, xfield, yfield, zfield, agg_func)
+        return point_aggregation(cvs, dataset, xfield, yfield, zfield, geometry_field, agg_func)
 
     elif geometry_type == 'line':
         return line_aggregation(cvs, dataset, zfield, agg_func)
@@ -106,7 +108,7 @@ def create_agg(source: MapSource,
         raise ValueError('Unkown geometry type for {}'.format(dataset['name']))
 
 
-def point_aggregation(cvs, data, xfield, yfield, zfield, agg_func):
+def point_aggregation(cvs, data, xfield, yfield, zfield, geometry_field, agg_func):
     """
     Compute a reduction by pixel, mapping data to pixels as points.
 
@@ -118,6 +120,8 @@ def point_aggregation(cvs, data, xfield, yfield, zfield, agg_func):
         The input datasource.
     xfield, yfield, zfield : str
         Column names for the x, y, and z coordinates of each point.
+    geometry_field: str
+        Column name for geometry field. If provided, the xfield and yfield arguments will be ignored
     agg_func : Reduction, optional
         Reduction to compute. Default is ``count()``.
 
@@ -126,10 +130,19 @@ def point_aggregation(cvs, data, xfield, yfield, zfield, agg_func):
     agg : xarray.DataArray
         The transformed datasource.
     """
+
     if zfield:
-        return cvs.points(data, xfield, yfield, getattr(ds, agg_func)(zfield))
+        if geometry_field:
+            return cvs.points(
+                data, agg=getattr(ds, agg_func)(zfield), geometry=geometry_field
+            )
+        else:
+            return cvs.points(data, xfield, yfield, getattr(ds, agg_func)(zfield))
     else:
-        return cvs.points(data, xfield, yfield)
+        if geometry_field:
+            return cvs.points(data, geometry=geometry_field)
+        else:
+            return cvs.points(data, xfield, yfield)
 
 
 def line_aggregation(cvs, data, zfield, agg_func):
@@ -256,6 +269,7 @@ def raster_aggregation(cvs, data, interpolate='linear', padding=0, agg_method=rd
 additional_transforms = {'hillshade': hillshade,
                          'quantile': quantile}
 
+
 def apply_additional_transforms(source: MapSource, agg: xr.DataArray):
     """
     Apply additional transforms over the data, which options could be
@@ -416,6 +430,7 @@ def shade_agg(source: MapSource, agg: xr.DataArray, xmin, ymin, xmax, ymax):
             print('Shade without Span')
             return tf.shade(agg, cmap=cmap, how=how)
 
+
 def to_raster(source: MapSource,
               xmin: float = None, ymin: float = None,
               xmax: float = None, ymax: float = None,
@@ -482,7 +497,7 @@ def render_map(source: MapSource,  # noqa: C901
                xmax: float = None, ymax: float = None,
                x: float = None, y: float = None,
                z: float = None,
-               height: int = None, width: int = None, ):
+               height: int = None, width: int = None):
     """
     Export a MapSource object to a map object.
 
@@ -542,23 +557,118 @@ def render_map(source: MapSource,  # noqa: C901
                                'y': np.linspace(ymin, ymax, height)},
                        dims=['x', 'y'])
         img = shade_agg(source, agg, xmin, ymin, xmax, ymax)
-        return img
+    else:
+        agg = create_agg(source, xmin, ymin, xmax, ymax, x, y, z, height, width)
 
-    agg = create_agg(source, xmin, ymin, xmax, ymax, x, y, z, height, width)
+        if source.span and isinstance(source.span, (list, tuple)):
+            agg = agg.where((agg >= source.span[0]) & (agg <= source.span[1]))
 
-    if source.span and isinstance(source.span, (list, tuple)):
-        agg = agg.where((agg >= source.span[0]) & (agg <= source.span[1]))
+        source, agg = apply_additional_transforms(source, agg)
+        img = shade_agg(source, agg, xmin, ymin, xmax, ymax)
 
-    source, agg = apply_additional_transforms(source, agg)
-    img = shade_agg(source, agg, xmin, ymin, xmax, ymax)
-
-    # apply dynamic spreading ----------
-    if source.dynspread and source.dynspread > 0:
-        img = tf.dynspread(img, threshold=1, max_px=int(source.dynspread))
+        # apply dynamic spreading ----------
+        if source.dynspread and source.dynspread > 0:
+            img = tf.dynspread(img, threshold=1, max_px=int(source.dynspread))
 
     return img
 
 
+def tile_to_disk(img, output_location, z=0, x=0, y=0, tile_format='png'):
+    """
+    Write a tile image to local disk
+
+    Parameters
+    ----------
+    img: xarray.DataArray
+        aggregation data array of the tile to write to image
+    x, y, z : float
+        The coordinates to be used to get the bounds inclusive space along the axis.
+    output_location: str
+        Path to write tile image
+
+    Returns
+    -------
+        None
+    """
+
+    tile_file_name = '{}.{}'.format(y, tile_format.lower())
+    tile_directory = os.path.join(output_location, str(z), str(x))
+    try:
+        os.makedirs(tile_directory)
+    except OSError as e:
+        import errno
+        if e.errno != errno.EEXIST:
+            raise
+    output_file = os.path.join(tile_directory, tile_file_name)
+
+    # save to local disk
+    print(f'Writing tile ({x, y, z}) to {output_file}')
+    img.save(output_file, tile_format)
+
+    return output_file
+
+
+def tile_to_s3(img, output_location, z=0, x=0, y=0, tile_format='png'):
+    """
+
+    Parameters
+    ----------
+    img
+    output_location
+    z
+    x
+    y
+    tile_format
+
+    Returns
+    -------
+
+    """
+
+    try:
+        import boto3
+    except ImportError:
+        raise ImportError('conda install boto3 to enable rendering to S3')
+
+    try:
+        from urlparse import urlparse
+    except ImportError:
+        from urllib.parse import urlparse
+
+    s3_info = urlparse(output_location)
+    bucket = s3_info.netloc
+    s3_client = boto3.client('s3')
+
+    tile_file_name = '{}.{}'.format(y, tile_format.lower())
+    key = os.path.join(s3_info.path, str(z), str(x), tile_file_name).lstrip('/')
+    output_buf = BytesIO()
+    img.save(output_buf, tile_format)
+    output_buf.seek(0)
+    s3_client.put_object(Body=output_buf, Bucket=bucket, Key=key, ACL='public-read')
+    return 'https://{}.s3.amazonaws.com/{}'.format(bucket, key)
+
+
+def render_tile(source, output_location, z=0, x=0, y=0, tile_format='png'):
+    agg = render_map(source, x=int(x), y=int(y), z=int(z), height=256, width=256)
+
+    if 0 in agg.shape:
+        return None
+
+    try:
+        from PIL.Image import fromarray
+    except ImportError:
+        raise ImportError('conda install pillow to enable rendering to local disk')
+
+    # flip since y tiles go down (Google map tiles)
+    img = fromarray(np.flip(agg.data, 0), 'RGBA')
+
+    if output_location.startswith('s3:'):
+        tile_to_s3(img, output_location, z, x, y, tile_format)
+    else:
+        # write to local disk
+        tile_to_disk(img, output_location, z, x, y, tile_format)
+
+
 def get_source_data(source: MapSource, simplify=None):
     """
     Get MapSource data and return as dict or GeoDataFrame depending on the geometry type.
@@ -576,7 +686,7 @@ def get_source_data(source: MapSource, simplify=None):
     gdf : GeoDataFrame or dict
         The Mapsource data
     """
-    if isinstance(source.data, spatialpandas.GeoDataFrame):
+    if isinstance(source.data, spd.GeoDataFrame):
         gdf = source.data.to_geopandas()
 
     elif isinstance(source.data, gpd.GeoDataFrame):
