@@ -97,6 +97,8 @@ class MapSource:
         Preload the data after the service started.
     force_recreate_overviews : bool, default=False
         For overviews to be recreated even if they already exist.
+    tiling: dict, default=None
+        Settings for saving tile images to an output location
     """
 
     def __init__(self,  # noqa: C901
@@ -134,7 +136,8 @@ class MapSource:
                  band=None,
                  attrs=None,
                  preload=False,
-                 force_recreate_overviews=False):
+                 force_recreate_overviews=False,
+                 tiling=None):
 
         if fields is None and isinstance(data, (gpd.GeoDataFrame)):
             fields = [geometry_field]
@@ -203,6 +206,7 @@ class MapSource:
         self.geometry_field = geometry_field
         self.band = band
         self.force_recreate_overviews = force_recreate_overviews
+        self.tiling = tiling
 
         self.is_loaded = False
         self.data = data
@@ -316,6 +320,8 @@ class RasterSource(MapSource):
         The map source object.
     """
 
+    source_type = 'raster'
+
     @property
     def load_func(self):
         return load_raster
@@ -342,6 +348,8 @@ class VectorSource(MapSource):
         The map source object.
     """
 
+    source_type = 'vector'
+
     @property
     def load_func(self):
         return load_vector
